/*
  My m25p40 Library
  version 0.1
  2021, electromicro.ru
  License: GNU GPL 2.1
*/

#ifndef MY25P40_CPP
#define MY25P40_CPP

#include "my25p40.h"

// Busy

void my25p40::checkBusy() {
  uint32_t stt = millis();
  while (busy()) { }
  uint32_t del = millis() - stt;
  if (del) {
    Serial.print(F(" (")); Serial.print(del); Serial.print(F(" ms)"));
  }
  Serial.println();
}

// Info

void my25p40::initInfo() {
  Serial.print(F("Init "));
  if (initialize()) {Serial.println(F("OK"));}
                     else {Serial.println(F("FAIL"));}
}

void my25p40::readInfo(uint32_t adr, String s) {
  Serial.print(F("Read #")); Serial.print(adr); Serial.print(F(": ")); Serial.println(s);
}

void my25p40::writeInfo(uint32_t adr, String s) {
  Serial.print(F("Write #")); Serial.print(adr); Serial.print(F(": ")); Serial.println(s);
}

// Erase

void my25p40::erase() {
  Serial.print(F("Full erase..."));
  chipErase();
  checkBusy();
}

uint32_t sectorAdr(byte i) {
  return FLASH_SECTOR_SIZE * i;
}

void my25p40::erase64K(byte sec) {
  uint32_t adr = sectorAdr(sec);
  Serial.print(F("Sector "));
  Serial.print(sec);
  Serial.print(F(" ("));
  Serial.print(adr);
  Serial.print(F(" + 64K) erase..."));
  
  blockErase64K(adr);
  checkBusy();
}

// Word

void my25p40::writeWord(uint32_t adr, word w) {
  byte bytes[2];
  bytes[0] = highByte(w);
  bytes[1] = lowByte(w);
  writeBytes(adr, bytes, 2);
}

word my25p40::readWord(uint32_t adr) {
  byte bytes[2];
  readBytes(adr, bytes, 2);
  word wordValue = (bytes[0] << 8) | bytes[1] & 0xFF;
  return wordValue;
}

// Unsigned long

ulb_t ulb;

void invertULong() {
  byte bytes[4];
  bytes[0] = ulb.b[0];
  bytes[1] = ulb.b[1];
  bytes[2] = ulb.b[2];
  bytes[3] = ulb.b[3];
  
  ulb.b[0] = bytes[3];
  ulb.b[1] = bytes[2];
  ulb.b[2] = bytes[1];
  ulb.b[3] = bytes[0];
}

void my25p40::writeULong(uint32_t adr, unsigned long ul) {
  ulb.ul = ul;
  invertULong();
  writeBytes(adr, ulb.b, 4);
}

unsigned long my25p40::readULong(uint32_t adr) {
  readBytes(adr, ulb.b, 4);
  invertULong();
  return ulb.ul;
}

// Int

uib_t uib;

void invertInt() {
  byte bytes[2];
  bytes[0] = uib.b[0];
  bytes[1] = uib.b[1];
  
  uib.b[0] = bytes[1];
  uib.b[1] = bytes[0];
}

void my25p40::writeInt(uint32_t adr, int i) {
  uib.i = i;
  invertInt();
  writeBytes(adr, uib.b, 2);
}

int my25p40::readInt(uint32_t adr) {
  readBytes(adr, uib.b, 2);
  invertInt();
  return uib.i;
}

// Long

lb_t lb;

void invertLb() {
  byte bytes[4];
  bytes[0] = lb.b[0];
  bytes[1] = lb.b[1];
  bytes[2] = lb.b[2];
  bytes[3] = lb.b[3];
  
  lb.b[0] = bytes[3];
  lb.b[1] = bytes[2];
  lb.b[2] = bytes[1];
  lb.b[3] = bytes[0];
}

void my25p40::writeLong(uint32_t adr, long lg) {
  lb.lg = lg;
  invertLb();
  writeBytes(adr, lb.b, 4);
}

long my25p40::readLong(uint32_t adr) {
  readBytes(adr, lb.b, 4);
  invertLb();
  return lb.lg;
}

// Float

flb_t flb;

void invertFloat() {
  byte bytes[4];
  bytes[0] = flb.b[0];
  bytes[1] = flb.b[1];
  bytes[2] = flb.b[2];
  bytes[3] = flb.b[3];
  
  flb.b[0] = bytes[3];
  flb.b[1] = bytes[2];
  flb.b[2] = bytes[1];
  flb.b[3] = bytes[0];
}

void my25p40::writeFloat(uint32_t adr, float fl) {
  flb.fl = fl;
  invertFloat();
  writeBytes(adr, flb.b, 4);
}

float my25p40::readFloat(uint32_t adr) {
  readBytes(adr, flb.b, 4);
  invertFloat();
  return flb.fl;
}

// Char array

void my25p40::writeChars(uint32_t adr, char chs[], byte len) {
  writeBytes(adr, chs, len);
}


void my25p40::readChars(uint32_t adr, char *buf, byte len) {
  readBytes(adr, buf, len);
  buf[len] = 0;
}

// String

void my25p40::writeString(uint32_t adr, String s) {
  int len = s.length() + 1;
  char char_array[len];
  strcpy(char_array, s.c_str());
  writeBytes(adr, char_array, len);
}

String my25p40::readString(uint32_t adr, byte len) {
  String s = "";
  char charArray[len];
  
  readBytes(adr, charArray, len);
  
  for (int i = 0; i < len; i++) {
    s += charArray[i];
  }
  
  return s;
}

#endif // #ifndef MY25P40_CPP
